# -*- coding: utf-8 -*-
# By making tests a python package you can split off any helper functions in
# separate source files, which can then be imported wherever you need them.
"""
The tests package for {{tmpl.project_name}}.
"""
