# -*- coding: utf-8 -*-

"""
Module {{tmpl.import_lib}}
==========================

A submodule for ...

"""

# your imports here ...

def greet(to=''):
	"""Say "Hello <to>!".
	
	:param str to: whom you want to say hello to.
	"""
	return f"Hello {to}!"